/**
 *  OnOffHeterogeneousAggregatedFlowGeneration : Generation de trafic voix pour N sources => flux aggreges
 *                              Ces flux sont differents: par leur duree, leurs caracteristiques => sources heterogenes.
 * 							Utiliser aggregatedScenarioGeneration(*) pour generer le scenario des flux aggreges
 * 							Utiliser flowsGeneration pour generer les scenarii des differents flots
 * 
 *
 * @author maria ladoue
 * @version 0.2 , 28/03/03 
 */

package multimedia.generation;

import java.io.*;
import java.util.*;

import statistic.distribution.*;
import utils.*;
import multimedia.codec.*;
import multimedia.model.*;


public class OnOffHeterogeneousAggregatedFlowGeneration extends GenericGeneration implements AudioCodecExample {
    
    protected File[] flowsFile;
    protected OnOffHeterogeneousAggregatedFlowModel model;
  
    protected String[] fN;  
    protected SortedMap tS;
    protected OnOffAggregatedFlow[] aof;
    protected TreeMap[] tM;
    protected int nbDifferentFlow;
    private double realRate = 0d;
    protected String serverName;
    protected int[] serverPort;
	protected short unit = BYTES;


	public OnOffHeterogeneousAggregatedFlowGeneration() {
		super();
	}

    /**
     * OnOffHeterogeneousAggregatedFlowGeneration 
     *
     *@param tpsSimul test duration in minutes
     *@param ooha define an  Heterogeneous On-Off Flows Aggregation
     *@param serverPort define the port number
     */
    public OnOffHeterogeneousAggregatedFlowGeneration ( int tpsSimul, OnOffHeterogeneousAggregatedFlowModel ooha, int serverPort) throws IOException { 
		this.tpsSimul = tpsSimul;
		model = ooha;
		nbDifferentFlow = model.getDifferentSrcTypeNumber();
		
		Calendar calendar = new GregorianCalendar();
		this.fileName = new String("aggOnOff_" + calendar.get(Calendar.YEAR) + (calendar.get(Calendar.MONTH)+1) + calendar.get(Calendar.DAY_OF_MONTH) + "_"  +calendar.get(Calendar.HOUR_OF_DAY) + "h" + calendar.get(Calendar.MINUTE));
		
		tS = new TreeMap();
		this.serverPort = new int[nbDifferentFlow];
		setPortNumber(-1,serverPort);
    }


	/**
     * To specify in addition a scenario file name and if you use a GUI
     *@param tpsSimul test duration in minutes
     *@param ooha define an  Heterogeneous On-Off Flows Aggregation
     *@param serverPort define the port number
     *@param fichierReq	scenario file name
     *@param graphic	To choose a graphical mode or not
     */
    public OnOffHeterogeneousAggregatedFlowGeneration ( int tpsSimul, OnOffHeterogeneousAggregatedFlowModel ooha, int serverPort, String fichierReq,  boolean graphic) throws IOException { 
		this( tpsSimul, ooha, serverPort);
		
		this.fileName = fichierReq;
		this.graphic = graphic;
    }

    /**
     * To specify a specific port number for each flow type
     *  
     *@param tpsSimul  test duration in minutes
     *@param ooha define an  Heterogeneous On-Off Flows Aggregation
     *@param fichierReq define the name of result file
     *@param serverPort define the port number
     *@param graphic determines if it will be used in a graphic environnement
     */
    public OnOffHeterogeneousAggregatedFlowGeneration ( int tpsSimul, OnOffHeterogeneousAggregatedFlowModel ooha, String fichierReq, String serverName, int[] serverPort, boolean graphic) throws IOException {	
		this( tpsSimul, ooha, serverPort[0], fichierReq, graphic);
		setPortNumber(serverPort);
    }

     /**
      * To determine scenarii of each source type which could be used by the generator
      *						
      *    Create a file format as follow:<BR>
      *  <i>Inter-packet time</i> /t  <i>packet size</i><BR>
      */
    public double flowScenariiGeneration() throws IOException {
    	
		MMPP2Model[] srcGroup = model.transformation();

		fN = new  String[nbDifferentFlow];
		tM = new TreeMap[nbDifferentFlow];
		aof = new OnOffAggregatedFlow[nbDifferentFlow];
		flowsFile = new File[nbDifferentFlow];
	
	    for (int i=0;i<nbDifferentFlow;i++) {
		tM[i] =  new TreeMap();
		
		flowsFile[i]= new File( fN[i] = new String("aggOnOff_"+ i) );
		
		aof[i] = new OnOffAggregatedFlow( srcGroup[i], tpsSimul, fN[i] , tM[i]);
		realRate += aof[i].scenarioGeneration();
		
		tS.putAll(aof[i].getTimeGeneration());
		tM[i] = null;
	    }
		return realRate;
	}	

    /**
     * To compute scenario of each source type with some informations
     *  It generates as much scenario as it's necessary
     * 
     *    Create a file format as follow:<BR>
     *    <i>day date</i> : <i>servername</i><BR>
     *    <i>modelName</i>    IAmoy : <i>tpsIP_codec</i> numero de port : <i>number</i> packet size :<i>size</i> bytes<BR>
     *    date: t + <i>testTime</i> s,  IA : <i>interpacketIime</i> : 0.000000 s <BR>
     *
     *@return the real throughput WARNING; we don't know unit
     */
    public double flowStatisticsGeneration( ) throws IOException {

		MMPP2Model[] srcGroup = model.transformation();

		fN = new  String[nbDifferentFlow];	
		tM = new TreeMap[nbDifferentFlow];
		aof = new OnOffAggregatedFlow[nbDifferentFlow];
		flowsFile = new File[nbDifferentFlow];
	
	    for (int i=0;i<nbDifferentFlow;i++) {
			tM[i] =  new TreeMap();
			fN[i] = new String("aggOnOff_"+ i);
			flowsFile[i]= new File(fN[i]);
		
			aof[i] = new OnOffAggregatedFlow( srcGroup[i], tpsSimul, fN[i], tM[i]);
		
			realRate += aof[i].statisticsGeneration();

			tS.putAll(aof[i].getTimeGeneration());
			tM[i] = null;
	    }
	    
	    return realRate;
	}

    /**
     * To compute an aggregated scenario  from generation of each scenario
     * WARNING: This create only one file
     *    Create a file format as follow:<BR>
     *  <i>Inter-packet time(in ms)</i> /t <i>size</i><BR>
     */
    public void scenarioGeneration() throws IOException {
		try {
			resultFile =  new File( "scenario.tmp" );
			this.fichierReq = FileTest.testExistent(resultFile.getName(),graphic);
		}
		catch (FileNotFoundException fnfe) {
			System.err.println("Name: " + fileName);
			fnfe.printStackTrace();
		}

		initializedData();
		
		System.out.println("Start: " + new Date().toString());
		flowScenariiGeneration();
		scenarioGenericGeneration( true);
		System.out.println("End: " + new Date().toString());			 
    }
    
    /**
     * To compute final scenario (with some informations) which is the aggregation of each scenarii pre-calculated
     * WARNING: This create only one file
     *    Create a file format as follow:<BR>
     * <i>day date</i> : <i>servername</i><BR>
     * <i>characteristic of the flow</i> \t theoritical throughput : <i>value</i><BR>
     * date (in seconds) \t Inter-Packet Time(in milliseconds) \t numero de port <BR>
     *<BR>
     * date : <i>date</i> s /t IA : <i>Inter-packet time</i> ms /t packet size : <i>size</i> Bytes<BR>
     *
     *@param serverName
     *@param portNumber
     */
	public void statisticsGeneration() throws IOException {

		String fileName = getResultFileName();
		resultFile = changeScenarioFileExtension( new File( fileName ), false);
		this.fichierReq = FileTest.testExistent( resultFile.getName(), graphic);
		
		initializedData();
		
		System.out.println("Start: " + new Date().toString());
		flowStatisticsGeneration();
		scenarioGenericGeneration( false);
		System.out.println("End: " + new Date().toString());			 	
    }
	
	protected double initializedData() throws IOException {
    	
		 MMPP2Model[] srcGroup = model.transformation();

		 tM = new TreeMap[nbDifferentFlow];
		 aof = new OnOffAggregatedFlow[nbDifferentFlow];
	
		 for (int i=0;i<nbDifferentFlow;i++) {
			 tM[i] =  new TreeMap();
			 aof[i] = new OnOffAggregatedFlow( srcGroup[i], tpsSimul, "",  tM[i]);	// ici fileName n'est pas important
			 realRate += aof[i].scenarioGenericGeneration( true, false); 
		
			 tS.putAll( aof[i].getTimeGeneration() );
			 tM[i] = null;
		 }
		 return realRate;
	 }
	
	
	/**
	 * @see multimedia.generation.GenericGeneration#scenarioGenericGeneration(boolean, java.lang.String, int)
	 */
	protected double scenarioGenericGeneration( boolean playable) throws IOException {
		
		int nbBytesTotal =0;	
		if(!playable) {               
			// Premiere ligne du scenario: date + nom_du_serveur
			fichierReq.write( new Date().toString() + "\n");
			fichierReq.write( "Requested documents : " + getDocumentNames() + "\n");
			fichierReq.write( this.toString() + "\t Theoritical throughput: " + meanRate() + " kb/s \n");
			fichierReq.write("Scenario format: date (in seconds) \t Inter-Packet Time(in milliseconds) \t packet size \n\n" );// Format du fichier scenario
		}
	
	// Pour le premier paquet
	if(playable) {
		fichierReq.write("0.0 \t" + (tS.get((tS.firstKey()))).toString() + "\n");
	}
	else {
		fichierReq.write("date: t + " + ((Double)(tS.firstKey())).doubleValue() + " s \t IA : " + 0.0  + " ms \t packet size : "+ (tS.get((tS.firstKey()))).toString() + " Bytes \n");
	}
	nbBytesTotal += ((Integer)tS.get((tS.firstKey()))).intValue();  // ajout 14/05/03
	
	double packet = ((Double)(tS.firstKey())).doubleValue();
	int packetNb = tS.size();	// correction 14/05/03
	tS.remove(tS.firstKey());
//	int packetNb = tS.size();

	for (int i=0;i<packetNb-1;i++) {
		if(playable) {
			fichierReq.write( (((Double)(tS.firstKey())).doubleValue()-packet)*1000 + "\t"  );
			fichierReq.write((tS.get((tS.firstKey()))).toString()+ "\n");
		}
		else {
			fichierReq.write("date: t + " + (tS.firstKey()).toString() + " s \t IA : " + (((Double)(tS.firstKey())).doubleValue()-packet)*1000 + " ms \t packet size : "+ (tS.get((tS.firstKey()))).toString() +" Bytes \n");
		}
	   	nbBytesTotal += ((Integer)tS.get((tS.firstKey()))).intValue();  // ajout 14/05/03
	    packet = ((Double)(tS.firstKey())).doubleValue();
	    tS.remove(tS.firstKey());    
	}
	
	fichierReq.flush();
	fichierReq.close();
	
	double debit = nbBytesTotal / tpsSimul; 	//ajout le 16/05/03
	
	if( !playable ) {            
	 String messageInfo ="";
	  messageInfo =  new String("Taille totale des documents transmit: " + nbBytesTotal + " Bytes.\n" +
		"Debit reel pour l'aggregation: " + MyNumberFormat.myDebitFormatter.format(getTheoriticalAggregatedFlowRate()) + " kb/s.\n" +
	   "Debit theorique pour l'aggregation: " + MyNumberFormat.myDebitFormatter.format( meanRate() ) + " kb/s soit: " + (meanRate()/1000) + " Mb/s");
	
	   MessageManager.informationMessage( messageInfo, graphic);
	}
	
	MessageManager.informationMessage("File created as: " + resultFile.getCanonicalPath(), graphic);

	return debit;
    }	// end of  scenarioGenericGeneration(boolean, String , int)

    /**
     * To set the port value for each flow
     *                 if idx=-1 it means that all values will be the same : serverPort
     *@param idx specify the port number
     *@param serverPort the value
     */
    public void setPortNumber(int idx, int serverPort) {
	if (idx == -1) {
	    for (int i=0;i<nbDifferentFlow;i++) {
		this.serverPort[i] = serverPort;
	    }
	}
	else {  this.serverPort[idx] = serverPort;	}
    }
    
    protected void setPortNumber( int[] serverPort ) {
 		if( serverPort.length == nbDifferentFlow ) {
 			for (int i=0;i<serverPort.length;i++) {	this.serverPort[i] = serverPort[i]; }
 		}
 		else { throw new IllegalArgumentException("You must specify a port number for each flow type"); }
    }   
    
    public void initializedSortedMap() { this.tS = new TreeMap(); }
    
    
     /**
      * Returns the port value which corresponds to the specified flow
      *                
      *@param idx specify the port number between 0 and nbDifferentFlow-1
      *@return the port value
      */
    public int getPort(int idx) { return serverPort[idx]; }    
 
 
 	public OnOffHeterogeneousAggregatedFlowModel getModel() { return model; }
 	public void setModel( OnOffHeterogeneousAggregatedFlowModel model ) { this.model = model; } 
 	
 
    /**
     * To compute mean of generated packet number per second <=> theoretical throughput
     *      mean = sum( mean<sub>i</sub> ) where i=1..nbDifferentFlow
     * 
     * @return mean rate in kb/s
     */
    public double meanRate() {
		double cumulDebit = 0;
		for (int i=0;i<nbDifferentFlow;i++) {
	    	cumulDebit += Math.round((model.getSpecificMMPP2Model(i)).meanRate());
		}
		return cumulDebit;
    }

	/**
	 * Returns throughput of aggregatedFlow in kb/s
	 * @return a double : this value is tranform to obtain a throughput in kb/s
	 */
	public double getTheoriticalAggregatedFlowRate() throws IOException { 
		if ( realRate==0 ) {
			flowScenariiGeneration(); 
		}
		double ans = 0;
		
		if(unit == BYTES) {
			ans = realRate/1000*8; // transform B => kb
		}
		else if (unit == KILOBYTES) {
			ans = realRate; // transform MB => kb
		}
		else if (unit == MEGABYTES) {
			ans = realRate*1000*8; // transform MB => kb
		}
		else if (unit == GIGABYTES) {
			ans = realRate*1000000*8; // transform GB => kb
		}
		return ans; 
	}
	
	/**
	 * Returns theoritical transmitted data amount during the generation phase in MB
	 * @return a double : this value is tranform to obtain an amount in MB
	 */
	public double getTheoriticalTransmittedDataAmount() throws IOException { 
		if ( realRate==0 ) {
			flowScenariiGeneration(); 
		}
		double ans = 0;
		
		if(unit == BYTES) {
			ans = realRate/1000000*tpsSimul*60; // transform B => MB	and mn => s
		}
		else if (unit == KILOBYTES) {
			ans = realRate/1000*tpsSimul*60; // transform MB => MB	and mn => s
		}
		else if (unit == MEGABYTES) {
			ans = realRate*tpsSimul*60; // transform MB => MB	and mn => s
		}
		else if (unit == GIGABYTES) {
			ans = realRate*tpsSimul*60; // transform GB => MB and mn => s
		}
		return ans; 
	}
	

    /**
     * Returns the number of flow defined for the test
     *@return the number of "On-Off Homogeneous" Flows
     */
    public int getFlowTypeNumber() { return nbDifferentFlow; }
    
	public void setFlowTypeNumber( int nbDifferentFlow ) { this.nbDifferentFlow = nbDifferentFlow; }
 
    /**
     * Method getResultFileName 
     *
     *@param idx specify the file number : idx = -1 => For an aggregated scenario file, idx =0...N it's to get a specific flow scenario file
     *@return the file path
     */
    public String getResultFileName(int idx) {
		if (idx==-1) { return resultFile.getPath(); }
		else {  return ( flowsFile[idx] ).getPath(); }
    }

    /**
     * Returns a description:  "Aggregation of " + nbDifferentFlow + " On-Off homogeneous sources :  " + model.getTypeSrcsNumbers()
     *@return characteristic of the  Aggregate Flow
     */
    public String toString() {	return new String("Aggregation of " + nbDifferentFlow + " On-Off homogeneous sources :  " + model.getTypeSrcsNumbers());  }
	

	/**
	 *  Returns a String which discribes the different flows as:
	 * 	 <i>model name or document name(if it exists)</i>	(<i>source number</i>)
	 * @return a string = testName
	 */
	public String getDocumentNames() {
		String reqDoc = "";
		for (int i=0;i<nbDifferentFlow;i++ ) {
			reqDoc += model.getSpecificMMPP2Model(i).getName() + " (" + model.getSourceNumber()[i] + ") \t ";
		}
		return reqDoc;
	}


	/**
	 * OnOffAggregatedFlow : internal class
	 * 
	 * @author mladoue
	 * @version 0.1
	 */
    class OnOffAggregatedFlow  {
	
		protected MMPP2Model model;
		protected int tpsSimul;
     	protected FileWriter fichierReq;
     	File flowScenario;
        private TreeMap tM;
		protected double throughput;
		protected String filName;


	/**
	 * Defines to use scenarioGeneration() methods
	 *@param modelName
	 *@param tpsSimul
	 *@param fichierReq
	 *@param tM
	 */
    OnOffAggregatedFlow ( MMPP2Model model, int tpsSimul, String scenarioName, TreeMap tM) throws IOException {		
		this.model = model;
		this.tpsSimul = tpsSimul;
		this.tM = tM;
		this.filName = scenarioName;
    }


    /**
     * To generate the file of a source type that will be used to create the aggregated flows file<BR>
     *		Scenario file format:
     *		Inter packet time(in micro-second): 0.000000<BR>
     */

    protected double scenarioGeneration() throws IOException {
		this.flowScenario = new File( fileName );
		this.fichierReq = new FileWriter( flowScenario );
		
    	return scenarioGenericGeneration(true);
    }
  
   /**
    * To generate a scenario file with information<BR>
    * Create a file format as follow:<BR>
    *    <i>date</i> : <i>serverName</i><BR>
    *    <i>modelName</i>    IAmoy : <i>tpsIP_codec</i> numero de port : <i>number</i> packet size :<i>size</i> bytes<BR>
    *    date: t + <i>testTime</i> s,  IA : <i>interpacketIime</i> : 0.000000 s <BR>
    *@return throughput  X/s
    */
	protected double statisticsGeneration() throws IOException {
		this.flowScenario = changeScenarioFileExtension( new File( fileName ), false);
		this.fichierReq = new FileWriter( flowScenario );
		
		return scenarioGenericGeneration( false);
	}
	

	protected double scenarioGenericGeneration( boolean playable) throws IOException {
		return scenarioGenericGeneration(false, playable);
	}	

	/**
	 * @param createData
	 * @param playable
	 * @param serverName
	 * @param serverPort
	 * @return
	 * @throws IOException
	 */
	protected double scenarioGenericGeneration( boolean createData, boolean playable) throws IOException {
		
		// Definition des variables
		double time = 0, s, r, sOld = 0;  // time is in  seconds
		int newTpsSimul = tpsSimul*60;
		int size = (model.getCodec()).getSize();
		int etat = model.getEtatDepart();
		double T1 = 0, T2=0;
		//double debitIns = 0;
		int nbPaquets1= 0, nbPaquets2= 0, nbPeriod = 0, nboctet = 0, cpt = 0;
		double tpsIP1=0, tpsIP2=0;
		
		
	if( !playable && !createData ) {
		// Premiere ligne du scenario: date + nom_du_serveur
		fichierReq.write( new Date().toString() + "\n");
		fichierReq.write( model.getName() + "\t IAmoy : " + (model.getCodec()).getTpsIP() + "\t packet size : " + size + " Bytes \n"); 	// Format du fichier scenario
		fichierReq.write("Scenario format: date (in seconds) \t Inter-Packet Time(in milliseconds) \t packet size(in bytes) \n\n" );// Format du fichier scenario
	}
	
	while (time < newTpsSimul) {
	    
	    if ( etat == 0 ) {
		s = time;
		T1 = model.calculT( model.getParam( etat, "beta") );
		tM.put( new Double(s), new Integer(size));
		
		if( !createData ) {
			if( playable ) {
				fichierReq.write( (s-sOld)*1000 + "\n");  // affiche le temps inter-packet
			}
			else {        
				fichierReq.write("date: t + " + MyNumberFormat.myTimeFormatter.format(s) + " s  \t IA: " + MyNumberFormat.myIAFormatter.format((s-sOld) * 1000) + " ms \n");
			}
		}	
		nbPaquets1++;
		sOld = s;

		if( model.getParam(0,"lambda") != 0d ) {
		    while( s < Math.min(time + T1, newTpsSimul) ) {
				tpsIP1 = model.calculT(model.getParam(etat,"lambda"));

			if ( tpsIP1 != 0d ) {
			    if ( (tpsIP1+s) < (time+T1) ) {    
				tM.put(new Double(s+tpsIP1),new Integer(size));
		
				if ( !createData ) {
					if( playable ) {
						fichierReq.write( ( (tpsIP1+s-sOld)*1000 ) + "\n");
					}
					else {        
						fichierReq.write("date : t + " +  MyNumberFormat.myTimeFormatter.format(tpsIP1+s) + " s \t IA: " + MyNumberFormat.myIAFormatter.format((s-sOld) * 1000) + " ms \n");
					}
				}	
				nbPaquets1++; // Pour calculer le debit moyen...
				s += tpsIP1;
			    }
			    else { break; }
			}
		    } // fin du deuxieme while
		}
		etat = 1;
		time += T1;	
	    } // fin du if
	    
	    else {
		r = time;
		T2 = model.calculT(model.getParam(etat,"alpha"));
		tM.put( new Double(r), new Integer(size));
		
		if ( !createData ) {
			if( playable ) {
				fichierReq.write( ( r - sOld )*1000 + "\n" );
			}
			else { 
				fichierReq.write("date: t + " +  MyNumberFormat.myTimeFormatter.format(r) + " s \t IA: " + MyNumberFormat.myIAFormatter.format((r-sOld) * 1000) + " ms \n");
			}
		}	
		nbPaquets2++;
		sOld=r;

		while ( r < Math.min( time + T2 , newTpsSimul)) {
		    tpsIP2 = model.calculT(model.getParam(etat,"lambda"));

		    if ( tpsIP2 != 0d ) { 
			if ( (tpsIP2+r) < (time+T2) ) {  
			    tM.put(new Double(r+tpsIP2),new Integer(size));
			    
				if ( !createData ) {
					if( playable ) {
						fichierReq.write( ((tpsIP2+r-sOld)*1000) + "\n");
					}
					else { 
			   			fichierReq.write("date : t + " +  MyNumberFormat.myTimeFormatter.format(r+tpsIP2) + "s \t IA: " + MyNumberFormat.myIAFormatter.format((tpsIP2+r-sOld)* 1000) + " ms \n");		
					}
				}
			    nbPaquets2++; // Pour calculer le debit moyen...
			    sOld = r;
			    r += tpsIP2;
			}
			else { r += T2; }
		    }
		}// fin du while
		etat = 0;
		time += T2;	
	    } // fin du else
	    
	    
	    if (cpt != 1 ) { cpt++;   }
	    else {
			nbPeriod++;
			// Calcul du debit moyen: debitmoy = ((n-1)* debitmoy(n-1) + debitIns(n))/n
			//debitIns = (modelName.getCodec()).getTaille()*(nbPaquets1*T1+nbPaquets2*T2)/(T1 + T2) ; // Donner le debit instantane pour une periode: T1 + T2
			//debit = ( (nbPeriod - 1)*debit + debitIns )/nbPeriod;
			nboctet += (nbPaquets1+ nbPaquets2)*(model.getCodec()).getSize();
			nbPaquets1 = 0;
			nbPaquets2 = 0;
			cpt = 0;
	    }// fin du else
	} // fin du while

	if ( !createData ) {
		fichierReq.flush();
		fichierReq.close();
	}
		
	double debit = nboctet/time;
	
	if ( !createData ) {
		if( !playable ) {
		 String messageInfo ="";
			messageInfo =  new String("Le debit moyen est de: " + debit/1000  + " kB/s, soit : " + debit*8/1000  + " kb/s.\n" +
		"Debit theorique: " + Math.round(model.meanRate()) + " kb/s.");

		MessageManager.informationMessage( messageInfo, graphic);
		}
	
		MessageManager.informationMessage("File created as: " + flowScenario.getCanonicalPath(), false);
	}
		
	return debit;
	
    }// fin du scenarioGenericGeneration(boolean, boolean)


    public TreeMap getTimeGeneration() { 	return tM;    }
	
    }  // end of OnOffAggregatedFlow
    
    
    /**
 	* Methode: Main pour tester la classe
 	*/
    public static void main (String[] args) throws IOException {
		OnOffModel[] srcType = { new OnOffModel(1,new ExponentialDistribution(0.1),new ExponentialDistribution(0.2),G726),
												new OnOffModel(1,new ExponentialDistribution(0.1),new ExponentialDistribution(0.2),G726)};
    	int[] nbTypeSrc = {2,2};
		OnOffHeterogeneousAggregatedFlowModel ooha = new OnOffHeterogeneousAggregatedFlowModel(srcType,nbTypeSrc);
		OnOffHeterogeneousAggregatedFlowGeneration agofg = new OnOffHeterogeneousAggregatedFlowGeneration(30,ooha,1260);
		agofg.scenarioGeneration();
		agofg.formatOutputFile("scenario.tmp");
    }
}


