/**
 * MMPP-2 : Modele MMPP-2 cas particulier d'un modele MMPP-N, qui permet de definir un modele IPP
 *          Interrupted Poisson Process.
 *          Il est tout a fait possible de definir un modele MMPP-2 valide a partir de l'implementation
 *          du modele MMPP-N.
 * 
 * @author maria ladoue
 * @version 0.1, 05/05/02
 */

package multimedia.model;

import statistic.distribution.*;
import multimedia.codec.*;

public class MMPP2Model extends MMPPNModel {

    //Definition des constructeurs
    /**
     * Default constructor
     */
    public MMPP2Model() {
		super();
    }

    /**
     * MMPP2Model 
     *
     * @param unEtatDepart it defines the state of the system at the start
     * @param unAlpha it's exponential distribution parameter (state 2)
     * @param unBeta it's exponential distribution parameter (state 1)
     * @param unLambda1 it's exponential distribution parameter which defines inter-packet time (state 1)
     * @param unLambda2 it's exponential distribution parameter which defines inter-packet time (state 2)
     * @param codec it's the associated Codec
     */
    public MMPP2Model( int unEtatDepart, double alpha, double beta, double lambda1, double lambda2, AudioCodec codec) {
		this();
		setEtatDepart( unEtatDepart );
		this.setParams( alpha, beta, lambda1, lambda2);
		this.codec = codec;
    }

    /**
     * MMPP2Model in the particular case of IPP Model
     *
     * @param unEtatDepart it defines the state of the system at the start
     * @param unAlpha it's exponential distribution parameter (state 2)
     * @param unBeta it's exponential distribution parameter (state 1)
     * @param unCodec it's the associated Codec
     * 
     * TJ: I'm not sure this constuctor should exists !
     */
   public MMPP2Model ( int unEtatDepart, double unAlpha, double unBeta,AudioCodec unCodec) {
		this( unEtatDepart,unAlpha,unBeta,Double.longBitsToDouble(0x7ff0000000000000L),1d/(unCodec.getTpsIP()),unCodec);
		name = new String("IPP");
		//lambda1 = Double.longBitsToDouble(0x7ff0000000000000L); On associe arbitrairement l'etat 1 a l'etat inactif: Double.longBitsToDouble(0x7ff0000000000000L)correspond a + l'infini en java
    }


    /**
     * To calculate mean of generated packet number per second <=> theoretical throughput
     *      mean = lambda1*alpha/(alpha+beta) + lambda2*beta/(alpha+beta)
     * 
     * @return mean rate in kb/s
     */
    public double meanRate() {
		if (lambda[0]!=Double.longBitsToDouble(0x7ff0000000000000L)) {
	    	return (beta[0]/(alpha[1]+beta[0])*lambda[1] + alpha[1]/(alpha[1]+beta[0])*lambda[0])*(getCodec().getSize())*8d/1000;
		}
		else {
	    	return (beta[0]/(alpha[1]+beta[0]))*lambda[1]*(getCodec().getSize())*8d/1000;
		}
    }
    
     public String getCompleteName() { return new String( name + " Model");  }
    
    /**
     * Method setParams : permit to set the parameters 
	 * @param alpha
	 * @param beta
	 * @param lambda1
	 * @param lambda2
	 * @throws IllegalArgumentException: to verify arguments are >0
	 */
	public void setParams( double alpha, double beta, double lambda1, double lambda2) throws IllegalArgumentException {
    	if ( alpha >0 && beta>0 && lambda1>0 && lambda2 > 0)   { 	
    		this.alpha[1] = alpha;   // alpha(state1) = 0; 
			this.beta[0] = beta;     // beta(state2) = 0; 
			lambda[0] = lambda1;
			lambda[1] = lambda2;
    	}
    	else {
    		throw new IllegalArgumentException("Le parametre d'une loi exponentielle doit etre positif.\n");
    	}
    }
    
  /**
   * Methode: Main pour tester la classe
   */

    public static void main (String[] args) {
    	// With this example, we show that theorical bitrate of 100 onoff model is 100 times the bitrate of a MMPP2 model.
		AudioCodec c1 = new AudioCodec(136,0.012);
		MMPP2Model ipp1 = new MMPP2Model(1,2.8409,1.5385,c1); // Exemple de modele IPP
		System.out.println(ipp1.getName() +" " + ipp1.getParam(0,"lambda") + " " + ipp1.getParam(1,"lambda"));
		System.out.println(ipp1.getParam(1,"alpha"));
		System.out.println("Debit theorique de ipp: " + ipp1.meanRate());

		OnOffModel[] srcType = { new OnOffModel(1,new ExponentialDistribution(2.8571429),new ExponentialDistribution(1.5384615),new AudioCodec(136,0.012) ) };
		int[] nbTypeSrc = { 100 };
		OnOffHeterogeneousAggregatedFlowModel ooha = new OnOffHeterogeneousAggregatedFlowModel(srcType,nbTypeSrc);

		MMPP2Model[] srcGroup = ooha.transformation();
	
		System.out.println("Debit theorique: " + srcGroup[0].meanRate());
		System.out.println("Debit theorique: " + new MMPP2Model(1,2.8409,1.5385,new AudioCodec(136,0.012)).meanRate());
    }	
} 
