/**
 * MMPPNModel : Modele MMPP-N+1 de modelisation d'une source audio<BR>
 *              For example: MMPP-3 => N=2 <BR>
 * 
 *              beta[0]     beta[1]      beta[2]                   beta[N-1]     beta[N]<BR>
 *
 *           0 --------- 1 --------- 2 --------- 3 - - - - - - N-1 --------- N<BR>
 *
 *  alpha[0]    alpha[1]    alpha[2]   alpha[3]                    alpha[N] <BR>
 *
 *                   lambda[1]    lambda[2]   lambda[3]                   lambda[N]  <BR>
 *
 *
 * @author maria ladoue
 * @version 0.1, 01/08/02
 */

package multimedia.model;


import statistic.distribution.*;
import multimedia.codec.*;

public class MMPPNModel  extends GenericAudioModel {

    // Definition des parametres 
    public double[] alpha;  // Parametres des lois exponentielles associes aux differents etats
    public double[] beta;   // Parametres des lois exponentielles associes aux differents etats
    public double[] lambda; // Parametres des lois exponentielles inter-paquet
	
    // Definition des constructeurs

    // Constructeur par defaut
    public MMPPNModel() {
    	this( 2, 1, 3.12, 0.78, new AudioCodec()); // in reality MMPP-N+1 so MMPP2 <=> etatcount= 1+1, etatDepart:  in this case it could be 0 or 1
    }

    /**
     * MMPPNModel : we use this constructor when a codec is specified
     *
     * @param unEtatCount defines N (im MMPP-N+1 modelName), it corresponds to the number of states
     * @param unEtatDepart Fix a start state
     * @param unAlpha parameter of exponential distributions
     * @param unBeta parameter of exponential distributions
     * @param unCodec it's the associated Codec which gives packet size and inter-packet time
     * @throws IllegalArgumentException
     */

    // La valeur de lambda est recupere via le codec
    public MMPPNModel(int unEtatCount,int unEtatDepart, double unAlpha, double unBeta,AudioCodec unCodec) throws IllegalArgumentException {

		name = new String("MMPPN");
		paramsCount = 7;
		setEtatCount(unEtatCount);
		setEtatDepart(unEtatDepart);
		this.codec = unCodec;
		this.setParams( unAlpha, unBeta, (1d/(unCodec.getTpsIP())));
//		this.setParam("alpha",unAlpha);
//		this.setParam("beta",unBeta);
//		this.setParam("lambda",1d/(unCodec.getTpsIP()));
    }


    /**
     * MMPP-N Model general description 
     *
     * @param unEtatCount it defines the number of states
     * @param unEtatDepart it defines the state of the system at the start
     * @param unAlpha it's a table of value which define exponential distribution parameter
     * @param unBeta it's a table of value which define exponential distribution parameter
     * @param unLambda it's a table of exponential distribution parameters which define inter-packet time (state 1 to N)
     * @param unCodec it's the associated Codec
     * @throws IllegalArgumentException
     */
    public MMPPNModel( int unEtatCount, int unEtatDepart, double[] unAlpha, double[] unBeta, double[] unLambda, AudioCodec unCodec) throws IllegalArgumentException {

		name = new String("MMPPN");
		paramsCount = 7;
		setEtatCount(unEtatCount);
		setEtatDepart(unEtatDepart);

		if ( unEtatCount != unAlpha.length || unAlpha.length !=  unBeta.length || unAlpha.length != unLambda.length) {
		    throw new IllegalArgumentException("MMPPNModel.constructor : you must define "+ unEtatCount + " alpha, beta and lambda values.");
		}
		else {
		  
		    this.alpha = unAlpha;
		    this.beta = unBeta;
		    this.lambda = unLambda; 
		}
    }
   
    /**
     * To set a param: alpha, beta or lambda
     *   alpha[0]=alpha(0)=0, alpha[1]=alpha(1)=alpha, ..., alpha[N]=alpha(N)=N*alpha
     *   beta[0]=beta(0)=N*beta, ..., beta[N]=beta(N)=0
     *   lambda[0]=lambda(0)=0,lambda[1]=lambda(1)=lambda, ...,lambda[N]=lambda(N)=N*lambda
     *
     * @param unParam
     * @param uneValeur
     * @throws IllegalArgumentException
     */

    public void setParam(String unParam, double uneValeur) throws IllegalArgumentException {
		try {
		    if ( uneValeur > 0 ) { 
			for ( int i=0 ; i < etatCount ; i++) {
			    if (unParam=="alpha") { alpha[i] = i*uneValeur; }
			    else if (unParam=="beta") { beta[i] = (this.etatCount-i-1)*uneValeur; }	    
			    else if (unParam=="lambda") { lambda[i] = i*uneValeur; }
			    else  { throw new IllegalArgumentException("Il n'y a pas de parametre " + unParam + " pour un " + getName() ); }
			}
		    }// fin du if
		    else { throw new IllegalArgumentException("Le parametre d'une loi exponentielle doit etre positif.\n"); }
		}
		catch (ArrayIndexOutOfBoundsException e) { 
		    System.out.println(e.getMessage());
		}
    }

    /**
     * Used to define all parameters: alpha, beta and lambda
     *
     * @param unAlpha we have:  alpha[0]=0  and  i>0 alpha[i]= i * unAlpha
     * @param unBeta we have:   beta[N]=0  and  i<etatCount    beta[i]= (etatCount-i) * unBeta
     * @param unLambda we have: lambda[0]=0  for i=1..N   lambda[i] = i * unLambda
     * @throws IllegalArgumentException
     */

    public void setParams( double unAlpha, double unBeta, double unLambda) throws IllegalArgumentException {
    	 
    	 // Initialise les valeurs de alpha, beta et lambda
		alpha = new double[etatCount];
		beta = new double[etatCount];
		lambda = new double[etatCount]; 
    	
		try {
		    if (( unAlpha > 0d) && ( unBeta > 0d) && ( unLambda > 0d)) {
			for ( int i=0 ; i < etatCount ; i++) {
			    alpha[i] = i*unAlpha;        // alpha[0]=alpha(etat 1->0)=0 => i=0 alpha=0
			    beta[i] = (this.etatCount-1-i)*unBeta;      // MMPP-N+1 modelName, beta[N]=beta(etat N -> N+1)=0  => i=N beta=0
			    lambda[i] = i*unLambda;
			}
		    }
		    else { throw new IllegalArgumentException("Le parametre d'une loi exponentielle doit etre positif."); }
		} // fin du try
		catch (ArrayIndexOutOfBoundsException e) { 
		    System.out.println(e.getMessage());
		}
    }
    
    


	/**
	 * Method calculT
	 * @see multimedia.GenericModel#calculT(double)
	 * @throws IllegalArgumentException
	 */
    public double calculT(double unParam)  throws IllegalArgumentException {
	  if ( unParam > 0d ) {
	      ExponentialDistribution loi = new ExponentialDistribution( unParam );
	      return loi.rand();
	  }
	  else { throw new IllegalArgumentException("Le parametre d'une loi exponentielle doit etre strictement positif:" + unParam); }
    }


    /**
     * getParam: use to get all parameters: alpha, beta and gamma 
     *
     * @param unEtat it contained between 0 and N (~etatCount-1)
     * @param param  the parameter name
     * @throws IllegalArgumentException
     */
    public double getParam(int unEtat,String param) throws IllegalArgumentException {
	try {

	    if ( ( unEtat >= 0 ) && ( unEtat < etatCount ) ) {
		if ( param == "alpha" ) { return alpha[unEtat]; } // Donne la valeur d'alpha associe a l'etat unEtat
		else if ( param== "beta" ) { return beta[unEtat]; }
		else if ( param == "lambda") { return lambda[unEtat]; }
		else { throw new IllegalArgumentException("Ce n'est pas un parametre valable pour un modele de type MMPP-N+1."); }
	    }
	    else { throw new IllegalArgumentException("Il n'y a pas de parametre " + param + " pour cet etat."); }
	} // fin du try

	catch(ArrayIndexOutOfBoundsException e) { 
	    System.out.println(e.getMessage());
	    e.printStackTrace();
	}
	return 0; // necessaire car la methode return un double
    }

    /**
     * meanRate : to calculate mean of generated packet number per second
     *      mean = N*1/InterPacketTime*Pon = N*lambda*Pon
     *           where N is the number of state 1...N for a MMPP-N+1, i.e. M=N+1 <=>  MMPP-M 
     * @return mean rate in kbits/s
     */
    public double meanRate() {
	// in the case od a aggregation of N On-Off flows
	return (etatCount-1)*getParamOn()*(1d/(getCodec().getTpsIP()))*(getCodec().getSize())*8d/1000;
	/**double result=0;
	   for (int i=0;i<getEtatCount();i++) {
	   result += (beta[i]+alpha[i])*lambda[i]/(beta[i]+alpha[i]);
	   }
	return result;*/
    }

	/**
	 * Method getParamOn.   to calculate Pon = beta/(alpha + beta)
     *  			where alpha = 1/Ton = alpha[1]  and beta = 1/Toff = beta[N-1]
	 * @return double
	 */
    public double getParamOn() {
		return getParam((this.etatCount-2),"beta")/(getParam(1,"alpha")+getParam((this.etatCount-2),"beta"));
    }

	/**
	* Recupere et fixe le nombre d'etat
	*/
	 public int getEtatCount() { return etatCount; }

	/**
	 * To position etatCount and to crate alpha, beta and lambda vector
	 * @see multimedia.model.GenericAudioModel#setEtatCount(int)
	 */
	public void setEtatCount( int unEtatCount ) throws IllegalArgumentException { 
		if ( unEtatCount > 1 ) {
			this.etatCount = unEtatCount; 
		}
		else { throw new IllegalArgumentException("N >= 2 : " + unEtatCount ); }
			
		alpha = new double[ etatCount ];
		beta = new double[ etatCount ];
		lambda = new double[ etatCount ];
	}

	/**
	 * Method getParameter.
	 * @param idx
	 * @return Object
	 * @throws IllegalArgumentException
	 */
    public Object getParameter(int idx) throws IllegalArgumentException {
 	switch (idx) {
	case 0: return new Integer(etatCount);
	case 1: return new Double(getParamOn());
	case 2: return new Double(getParam(1,"alpha"));  // to get alpha
	case 3: return new Double(getParam(etatCount-2,"beta"));  // to get beta
	case 4: return new Double(getParam(1,"lambda"));  // to get lambda in the case of MMPP-N+1 corresponds to an aggregation of N IPP/On-Off sources
	default: throw new IllegalArgumentException("MMPPNModel.getParameter : it's not a valid parameter");
	}
    }

   public String getCompleteName() { return new String(name + "(" + (etatCount) + ")" + " Model");   }
 

    /**
     * Methode: Main pour tester la classe
     */

    public static void main (String[] args) {	
	//MMPPNModel m1 = new MMPPNModel();
	//m1.setParams(0.1,0.2,0.3);
	//MMPPNModel m2 = new MMPPNModel(3,2,0.2,0.1,0.3);
	//System.out.println(m1.getEtatDepart() + " " + m1.getEtatCount());
	//System.out.println(m2.getEtatDepart() + " " + m2.getEtatCount() + " " + m2.calculT(Double.longBitsToDouble(0x7ff0000000000000L)));
	//AudioCodec c1 = new AudioCodec(200,0.25);
	//MMPPNModel m3 = new MMPPNModel(20,1,3.12,0.78,c1);
	//System.out.println(m3.getEtatCount());
	//System.out.println(m3.getEtatDepart());
	//System.out.println(m3.getName() + " " + m3.getParam(5,"lambda") + " " + m3.getParam(1,"lambda"));
	//OnOffModel[] models = { new OnOffModel(2,new ExponentialDistribution(0.2),new ExponentialDistribution(0.1),c1),new OnOffModel(1,new ExponentialDistribution(2.8409),new ExponentialDistribution(1.5385),c1)};
	//int[] weights = {2,3};
	//OnOffHeterogeneousAggregate ooha = new OnOffHeterogeneousAggregate(models,weights);
    
	//MMPP2Model[] srcGroup = ooha.tranformation();
	//System.out.println(srcGroup.length);
	//System.out.println((srcGroup[0]).getParam(1,"beta"));
	//System.out.println((srcGroup[0]).getEtatCount());
	//MMPPNModel mmpp225 = new MMPPNModel(225,10,2.8409,1.5385,new AudioCodec(200,0.05));
	MMPPNModel mmpp20 = new MMPPNModel(20,0,3.12,0.78,new AudioCodec(1000,0.25));
	System.out.println(mmpp20.getName());
	System.out.println(mmpp20.getCompleteName());
	System.out.println(mmpp20.getEtatCount());
	//System.out.println(mmpp20.getCodec().getTpsIP());
	//System.out.println(mmpp20.getParam(1,"lambda"));
	System.out.println(mmpp20.getParam(mmpp20.getEtatCount()-2,"beta"));
	System.out.println(mmpp20.getParam(1,"alpha"));
	System.out.println(mmpp20.getParameter(4));
	System.out.println(mmpp20.getParamOn());
	System.out.println(mmpp20.meanRate());
    
    }
 
}
