/**
 * InverseGaussianDistribution : Definition de la loi Inverse Gaussienne
 *
 * @author maria ladoue
 * @version 0.1, 30/08/02 
 */

package statistic.distribution;
  
import java.lang.Math;    

import statistic.*;

public class InverseGaussianDistribution extends GenericContinuousDistribution implements RandomGenerator {

    /**    
     * Equation de la densite:(probability density)<BR> 
     *  f(x) = sqrt(sigma/(2*pi*x^3))*exp(-sigma*(x-mu)^2/(2*mu^2*x))) pour x>0
     */

    private static int MU = 0;
    private static int SIGMA  = 1;

    public InverseGaussianDistribution () {
        paramsCount = 2;
        name = new String("InvGaussian");
        paramNames = new String[paramsCount];
        paramNames[MU] = new String("mu");
        paramNames[SIGMA] = new String("sigma");
        params = new double[paramsCount];
        params[MU] = 1.0d;
        params[SIGMA] = 1.0d;
    }

    public InverseGaussianDistribution (double mu, double sigma) throws IllegalArgumentException {
        this();
        if (sigma > 0 && mu > 0) {
            params[MU] = mu;
            params[SIGMA] = sigma;
        }
	else {
            throw new IllegalArgumentException("InverseGaussianDistribution.constructor : sigma and mu must be greater than 0");
        }
    }

    public InverseGaussianDistribution(double[] params) throws IllegalArgumentException {
        this();
        if (hasValidParam(params))
        	setParams(params);
        else
			throw new IllegalArgumentException("InverseGaussianDistribution.constructor : sigma and mu must be greater than 0");
    }

	public boolean hasValidParam(double[] desParams) {
		if (desParams[SIGMA]  > 0 && desParams[MU]  > 0)
			return true;
		else
			return false;
	}

    public void setParams(double[] par) throws IllegalArgumentException {
        if (par.length >= paramsCount && par[SIGMA] > 0 && par[MU] > 0) {
            params[MU] = par[MU];
            params[SIGMA] = par[SIGMA];
	} 
	else {
            throw new IllegalArgumentException("InverseGaussianDistribution.setParams : sigma and mu must be greater than 0");
        }
    }

    public void setParam(int idx, double p) throws IllegalArgumentException {
        if (idx < paramsCount && p > 0) {
            params[idx] = p;
        } 
	else {
            throw new IllegalArgumentException();
        }
    }
    /**
     * Returns the value of the probability distribution function at x
     */
    public double pdf(double x) {
        if (x > 0) {
        return
            Math.sqrt(params[SIGMA] / (2 * Math.PI * Math.pow(x, 3.0d))) /
            Math.exp(params[SIGMA] *
                     Math.pow((x - params[MU])/params[MU], 2.0d) / (2 * x));
        }
	else {
            return 0d;
        }
    }

   /**
     * Returns the value of the cumulative distribution function at x
     */
    public double cdf(double x) {
        if (x > 0) {
            return
                normalStandardCdf(Math.sqrt(params[SIGMA]/x) *
                                  (-1.0d + (x/params[MU]))) +
                Math.exp(2.0d * params[SIGMA]/params[MU]) *
                normalStandardCdf(Math.sqrt(params[SIGMA]/x) *
                                  (-1.0d - (x/params[MU])));
        }
	else {
            return 0d;
        }
    }

    /**
     * Return the value of Inverse Function at x
     * @param x with x>=0 and x<=1
     */
    public double inverseCdf(double x) {
	return 0d; // to Do
    }

    /**
     * Returns the mean of the distribution
     */
    public double mean() {
        return params[MU];
    }

    /**
     * Returns the variance of the distribution
     */
    public double var() {
        return (Math.pow(params[MU], 3.0d) / params[SIGMA]);
    }

    public double rand() {
        double y = Math.pow(random.nextGaussian() , 2.0d);
        double x1 = params[MU] +
            ((Math.pow(params[MU], 2.0d) * y) / (2.0d * params[SIGMA])) -
            ((params[MU] / (2.0d * params[SIGMA])) *
             Math.sqrt(4.0d * params[MU] * params[SIGMA] * y +
                       (Math.pow(params[MU], 2.0d)*Math.pow(y, 2.0d))));

        if (random.nextDouble() <= (params[MU] / (params[MU] + x1))){
            return x1;
        }
	else {
            return Math.pow(params[MU], 2.0)/x1;
        }
    }
	
	public String getEquation() {
		return new String(" f(x) = sqrt(sigma/(2*pi*x^3))*exp(-sigma*(x-mu)^2/(2*mu^2*x))) for x>0");
	}

}
