/**
 * ParetoDistribution : Definition de la loi de Pareto
 *
 * @author maria ladoue
 * @version 0.1, 06/08/02 
 */

package statistic.distribution;

import java.lang.Math;

import statistic.*;

public class ParetoDistribution extends GenericContinuousDistribution implements RandomGenerator  { 
    /**
     * Equation de la densite:	
     *  f(x)= beta*lambda^beta*x^(-(beta+1)) pour x>=lambda, lambda>0, beta>0
     */
  
	private static int LAMBDA=0, BETA=1; // Definition de la "place" des parametres dans le tableau params[]

    /**
     * Definition des constructeurs
     */

    public ParetoDistribution() {
		paramsCount = 2;
		name = new String("Pareto");
		params = new double[paramsCount]; // initialisation du tableau des valeurs des parametres
		paramNames = new String[paramsCount]; // initialisation du tableau des noms des parametres
		params[LAMBDA] = 1.0d;
		params[BETA] = 0.5d;
		paramNames[LAMBDA] = new String("lambda");
		paramNames[BETA] = new String("beta");
    }

    public ParetoDistribution(double unLambda, double unBeta ) throws IllegalArgumentException {
		this();
		if (unLambda >0 && unBeta >0) {
			setParam(LAMBDA,unLambda);
			setParam(BETA,unBeta);
		}
		else
			throw new IllegalArgumentException("ParetoDistribution.constructor : lambda <=0 || beta <=0");
    }

    public ParetoDistribution(double[] desParams) throws IllegalArgumentException {
		this();
		if (hasValidParam(desParams))
			setParams(desParams);
		else
			throw new IllegalArgumentException("ParetoDistribution.constructor : lambda <=0 || beta <=0");
    }

	public boolean hasValidParam(double[] desParams) {
		if (desParams[LAMBDA]>0 && desParams[BETA]>0)
			return true;
		else
			return false;
	}

    // Definitions de quelques methodes
    
    /**
     * Returns the value of the probability distribution function at x
     */
    public double pdf(double x) {
        if (x >= params[LAMBDA]) {
            return  ( params[BETA] *                     
		      Math.pow(params[LAMBDA] , params[BETA]) *
		      Math.pow( x, - 1d * (params[BETA] + 1))); // to perform the division
        } 
	else { return 0.0d; }
    }
 
    /**
     * Returns the value of the cumulative distribution function at x
     */
    public double cdf(double x) {
        if (x > params[LAMBDA]) {
            return (1 - Math.pow(params[LAMBDA] / x, params[BETA]));
        } 
	else { return 0.0d; }
    }

    /**
    * Return the value of Inverse Function at x
    * @param x with x>=0 and x<=1
    */
    public double inverseCdf(double x) {
	if (x>=0 && x<=1) {
	    return ( params[LAMBDA] * Math.pow( 1d - x , -1d/params[BETA] ) );
	}
	else { return 0d; }
    }
    
    /**
     * Returns the mean of the distribution
     */
    public double mean() {
        if (params[BETA] > 1) {
            return (params[LAMBDA] * params[BETA] / (params[BETA] - 1d));
        } 
	else { return Double.POSITIVE_INFINITY; }
	
    }

    /**
     * Returns the variance of the distribution
     */
    public double var(){
        if (params[BETA] > 2) {
            return ( Math.pow(params[LAMBDA], 2.0d) * params[BETA] / 
		     (( params[BETA] - 2) * Math.pow(params[BETA] - 1, 2.0d)) );
        } 
	else {  return Double.POSITIVE_INFINITY; }
    }


    public double rand() {
		return (1/params[LAMBDA] * Math.pow( 1.0d - random.nextDouble() , -1.0d/params[BETA] ));
    }

	public String getEquation() {
		return new String("f(x)= beta*lambda^beta*x^(-(beta+1)) with x>=lambda, lambda>0, beta>0");
	}

    /**
     * Methode: Main pour tester la classe
     */

    public static void main (String[] args) {
	ParetoDistribution loiPar1 = new ParetoDistribution(1,2);
	//double[] desParams = {0.2,0.5};
	//ParetoDistribution loiPar2 = new ParetoDistribution(desParams);

	//loiPar2.setParam(1,0.6);
	//System.out.println(loiPar2.getParam(1));

	for (int i=0;i<5000;i++) {
	    System.out.println(loiPar1.rand());
	}
    } 

}
