/**
 * WeibullDistribution : Definition de la loi de Weibull
 *
 * @author maria ladoue
 * @version 0.1, 06/08/02
 */
package statistic.distribution; // Permet de specifier que cette classe appartient au package Statistique

import JSci.maths.SpecialMath;

import statistic.*;

import java.lang.Math;


//TODO: TJ, check if a >0 or a > 1 ???

public class WeibullDistribution extends GenericContinuousDistribution
    implements RandomGenerator {
    /**
     * Equation de la densite: <BR>
     *    f(x)=a*lambda*x^(a-1)*exp(-lambda*x^a) pour x>0, avec a>0, lambda>0
     *       where lambda = risk variable; must be > 0
     *             a = explanatory variable; must be >= 1
     *  If a=1,the Weibull is the same as Exponential distribution with lambda=1/lambda
     */
    private static int A = 0;
    private static int LAMBDA = 1;

    /*
     * Definition des constructeurs
     */
    public WeibullDistribution() {
        paramsCount = 2;
        name = new String("Weibull");

        paramNames = new String[paramsCount];
        paramNames[A] = new String("a");
        paramNames[LAMBDA] = new String("lambda");

        params = new double[paramsCount];
        params[A] = 1.0d;
        params[LAMBDA] = 1.0d;
    }

    public WeibullDistribution(double unA, double unLambda)
        throws IllegalArgumentException {
        this();

        if ((unA > 0) && (unLambda > 0)) {
            setParam(A, unA);
            setParam(LAMBDA,unLambda);
        } else {
            throw new IllegalArgumentException(
                "WeibullDistribution.constructor: Lambda and A must be greater than 0.");
        }
    }

    public WeibullDistribution(double[] desParams)
        throws IllegalArgumentException {
        this();
		System.out.println("ok");
        if (hasValidParam(desParams)) {
            setParams(desParams);
        } else {
            throw new IllegalArgumentException(
                "WeibullDistribution.constructor: Lambda and A must be greater than 0.");
        }
    }

    /**
     * Check that the parameters of the distribution are ok.
     * A >0 && lambda >=0
     * @param desParams distribution parameters
     * @return true if they match the requirements (see class descrption)
     */
    public boolean hasValidParam(double[] desParams) {
        if ((desParams[0] > 0) && (desParams[1] >= 0)) {
            return true;
        } else {
            return false;
        }
    }

    // Definitions de quelques methodes

    /**
     * Returns the value of the probability distribution function at x
     */

    // Densite
    public double pdf(double x) {
        if (x > 0) {
            return (((params[A] * params[LAMBDA]) * Math.pow(x, params[A] -
                1.0d)) / Math.exp(params[LAMBDA] * Math.pow(x, params[A])));
        } else {
            return 0d;
        }
    }

    /**
     * Returns the value of the cumulative distribution function at x
     */

    // Fonction de repartition: F(x)
    public double cdf(double x) {
        if (x > 0) {
            return (1.0d -
            Math.exp(-1.0d * params[LAMBDA] * Math.pow(x, params[A])));
        } else {
            return 0d;
        }
    }

    /**
     * Return the value of Inverse Function at x
     * @param x with x>=0 and x<=1
     */
    public double inverseCdf(double x) throws IllegalArgumentException {
        // Attention Math.log: correspond bien au log en base e (i.e log neperien)
        if ((x >= 0) && (x <= 1)) {
            return (Math.pow(-Math.log(1 - x) / params[LAMBDA], 1 / params[A]));
        } else {
            throw new IllegalArgumentException(
                "WeibullDistribution.inverseCdf : x must be >=0 and <=1");
        }
    }

    /**
     * Returns the mean of the distribution
     */
    public double mean() {
        return Math.pow(params[LAMBDA], -1d / params[A]) * SpecialMath.gamma(1 +
            (1.0 / params[A]));
    }

    /**
     * Returns the variance of the distribution
     */
    public double var() {
        double tmp;
        tmp = Math.pow(params[LAMBDA], -2.0d / params[A]) * (SpecialMath.gamma(1 +
                (2.0d / params[A])) -
            Math.pow(SpecialMath.gamma(1d + (1d / params[A])), 2.0d));

        return tmp;
    }

    public double rand() {
        return params[A] * Math.pow(Math.log(1.0 / random.nextDouble()),
            (1 / params[LAMBDA]));
    }

    public String getEquation() {
        return new String(
            "f(x)=a*lambda*x^(a-1)*exp(-lambda*x^a) for x>0, with a>0, lambda>0");

        //         where lambda = risk variable; must be > 0 
        //             a = explanatory variable; must be >= 1 
        //  If a=1,the Weibull is the same as Exponential distribution with lambda=1/lambda")
    }

    /**
     * Methode: Main pour tester la classe
     */
    public static void main(String[] args) {
        WeibullDistribution loiWei1 = new WeibullDistribution(0.2d, 0.5d);

        //double[] desParams = {0.2,0.5};
        //WeibullDistribution loiWei2 = new WeibullDistribution(desParams);
        for (int i = 0; i < 100; i++) {
            System.out.println(loiWei1.rand());
        }
    }
}
